# http://www.voidspace.org.uk/python/odict.html
# http://groups.google.com/group/comp.lang.python/browse_frm/thread/5ac716f9ad14cc04
# mhlib.IntSet is like our RangeValueSet
# http://code.google.com/p/pyquivalence/
# http://code.google.com/p/intervalmap/

__all__ = ['ReadonlySortedDict', 'RangeValueMap']


class ReadonlySortedDict(object):

    def __init__(self, items):
        self._lst = list(items)
        self._dct = dict(self._lst)

    def __len__(self):
        return len(self._lst)

    def get(self, key, default=None):
        return self._dct.get(key, default)

    def __getitem__(self, key):
        if isinstance(key, int):
            return self._lst[key][1]
        else:
            return self._dct[key]

    def __iter__(self):
        return iter(self._lst)

    def keys(self):
        return [a for a,v in self._lst]

    def values(self):
        return [v for a,v in self._lst]

    def items(self):
        return self._lst




class RangeValueMap(object):
    '''Mapping from range of keys to values

       Range is tuple of two keys taken from set of discrete values. These
       values must be comparable and must support plus/minus operations.
       There is optional parameter 'resolution' with default value 1,
       that defines minimal difference between any two keys.

       RangeValueMap is efficient for small number of large ranges where
       builtin dict would produce many items.

            >>> rv = RangeValueMap(((0,9),'a'), ((10,19),'b'))
            >>> rv.setrange(20,29,'c')
            >>> rv.get(5)
            'a'
            >>> rv.get(27)
            'c'
            >>> rv.get(37)
            >>> rv.get(37, 'n')
            'n'
            >>> list(rv.items())
            [((0, 9), 'a'), ((10, 19), 'b'), ((20, 29), 'c')]
            >>> list(rv.items(5,15))
            [((5, 9), 'a'), ((10, 15), 'b')]
            >>> list(rv.items(5,15, False))
            [((0, 9), 'a'), ((10, 19), 'b')]
            >>> rv.minkey()
            0
            >>> rv.maxkey()
            29
            >>> rv.setrange(4,4,'a4')
            >>> list(rv.items(0,10))
            [((0, 3), 'a'), ((4, 4), 'a4'), ((5, 9), 'a'), ((10, 10), 'b')]


       RangeValueMap is tested with int's and datetime.date's (resolution
       is day i.e. datetime.timedelta(1))

    '''

    resolution = 1
    cmpvalue   = staticmethod(lambda o: o)
    join       = staticmethod(lambda a,b: a)

    def __init__(self, *values, **kw):
        for a in 'resolution', 'cmpvalue', 'join':
            if a in kw:
                setattr(self, a, kw[a])
        self._ranges = []
        sr = self.setrange
        for k,v in values:
            if isinstance(k, tuple):
                s, e = k
            else:
                s = e = k
            sr(s,e,v)

    def copy(self):
        kw = {}
        for a in 'resolution', 'cmpvalue', 'join':
            if a in self.__dict__:
                kw[a] = self.__dict__[a]
        cp = self.__class__(*self.items(), **kw)
        return cp

    def setrange(self, start, end, value):
        if start > end:
            raise ValueError('start after end', start, end, value)
        ranges = self._ranges
        # empty container
        if not ranges:
            ranges.append((start, end, value))
            return
        res = self.resolution

        def new_split(start, end, value):
            riter = iter(ranges)
            current = riter.next()
            # yield all elements that are lesser than new
            while start > current[1]:
                yield current
                try:
                    current = riter.next()
                except StopIteration:
                    # all values are lesser than new
                    yield (start, end, value)
                    return
            # yield old overlayed part that is lesser than new
            if current[0] < start:
                yield (current[0], start-res, current[2])
            # eat all elements that are replaced wholy with new item
            while current[1] <= end:
                try:
                    current = riter.next()
                except StopIteration:
                    break
            # yield new value
            yield (start, end, value)
            # yield old overlayed parth that is greated than new
            if current[0] <= end < current[1]:
                yield (end+res, current[1], current[2])
            # pending current is whole after new value
            if current[0] > end:
                yield current
            # yield rest
            for current in riter:
                yield current

        # join neighbour ranges with the same value
        new = []
        split_iter = new_split(start, end, value)
        current = split_iter.next()
        comp = self.cmpvalue
        join = self.join
        for next in split_iter:
            if comp(current[2]) == comp(next[2]) \
            and current[1] == (next[0]-res):
                current = (current[0], next[1], join(current[2], next[2]))
            else:
                new.append(current)
                current = next
        new.append(current)
        self._ranges = new

    def delrange(self, start, end, split=True):
        raise NotImplementedError

    def get(self, key, default=None):
        # XXX bisect
        for s,e,v in self._ranges:
            if key >= s and key <= e:
                return v
        return default

    def getitem(self, key, default=((None,None),None)):
        for item in self.items(start=key, end=key, split=False):
            return item
        return default

    def items(self, start=None, end=None, split=True):
        ranges = self._ranges
        if not ranges:
            return
        if start is None and end is None:
            for s,e,v in ranges:
                yield (s,e),v
            return
        if start is None:
            start = self.minkey()
        if end is None:
            end = self.maxkey()
        # XXX bisect
        if split:
            for s,e,v in self._ranges:
                if e < start or s > end:
                    continue
                yield (max(s, start), min(e, end)), v
        else:
            for s,e,v in self._ranges:
                if e < start or s > end:
                    continue
                yield (s, e), v

    def keys(self, start=None, end=None, split=True):
        for k,v in self.items(start, end, split):
            yield k

    def values(self, start=None, end=None, split=False):
        for k,v in self.items(start, end, False):
            yield v

    def minkey(self, default=None):
        ranges = self._ranges
        if not ranges:
            return default
        else:
            return ranges[0][0]

    def maxkey(self, default=None):
        ranges = self._ranges
        if not ranges:
            return default
        else:
            return ranges[-1][1]

    def update(self, other):
        if not isinstance(other, self.__class__):
            raise ValueError('Instance of %s class required' % self.__class__)
        sr = self.setrange
        for s,e,v in other._ranges:
            sr(s, e, v)
