#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''pomocnici na cteni vyskopisneho ESRI gridu'''


class ESRIHeader(object):

    def __init__(self, **kw):
        self.ncols = None
        self.nrows = None
        self.xllcorner = None
        self.yllcorner = None
        self.cellsize = None
        self.nodata_value = -9999

        self.rec_len = 7
        self.min_value = None
        self.max_value = None

        for (a,v) in kw.iteritems():
            setattr(self, a, v)

    def get_rowlen(self):
        return (self.rec_len + 1) * self.ncols
    row_len = property(get_rowlen)

    def get_xmin(self):
        if self.xllcorner is None:
            raise ValueError('xllcorner not set')
        return self.xllcorner
    xmin = property(get_xmin)

    def get_xmax(self):
        if self.ncols is None:
            raise ValueError('ncols not set')
        if self.cellsize is None:
            raise ValueError('cellsize not set')
        return self.xmin + self.cellsize*(self.ncols-1)
    xmax = property(get_xmax)

    def get_ymin(self):
        if self.yllcorner is None:
            raise ValueError('yllcorner not set')
        return self.yllcorner
    ymin = property(get_ymin)

    def get_ymax(self):
        if self.nrows is None:
            raise ValueError('nrows not set')
        if self.cellsize is None:
            raise ValueError('cellsize not set')
        return self.ymin + self.cellsize*(self.nrows-1)
    ymax = property(get_ymax)


    @classmethod
    def fromAsciiGridFile(klass, gf):
        '''create ESRIHeader object from ascii grid file'''
        gf.seek(0)
        hd = {}
        for i in range(6):
            l = gf.readline().split()
            hd[l[0].lower()] = l[1]
        for f in ['ncols', 'nrows', 'nodata_value']:
            hd[f] = int(hd[f])
        for f in ['xllcorner', 'yllcorner', 'cellsize']:
            hd[f] = float(hd[f])
        return klass(**hd)

    @classmethod
    def fromIniFile(klass):
        from wm4.api import config
        hd = {}
        for k in ('xllcorner', 'yllcorner', 'cellsize'):
            hd[k] = float(config.Property('grid.%s' % k)())
        for k in ('ncols', 'nrows'):
            hd[k] = int(config.Property('grid.%s' % k)())
        return klass(**hd)

    def relativeCoords(self, coords):
        '''row and column of given coords in gridfile'''
        rx = int((coords[0] - self.xmin) / self.cellsize)
        ry = int((-1 * (coords[1] - self.ymax)) / self.cellsize)
        return (rx, ry)





def search(dbf, hd, coord):
    '''nalezeni hodnoty v souboru dbf, hlavicku hd pro souradnice (x,y)'''
    x,y = coord
    if x < hd.xmin or x > hd.xmax or y < hd.ymin or y > hd.ymax:
        return None

    x,y = hd.relativeCoords(coord)
    dbf.seek(y*hd.row_len + x*(hd.rec_len+1))
    return float(dbf.read(hd.rec_len))





def parse_data(gf, hd):
    '''vraci postupne jednotlive radky z gridu jako tuple (x, y, hodnota)
        na vstupu je soubor s gridem a rozparsovana hlavicka
        vedlejsim efektem je zapsani maximalni a minimalni hodnoty do hlavicek
    '''
    # preskoceni hlavicky
    gf.seek(0)
    for i in range(6): gf.readline()

    rows_processed = 0
    y = hd.ymax
    max_alt = None
    min_alt = None
    for row in gf:
        d_row = []
        x = hd.xmin
        for col in row.split():
            alt = float(col)
            d_row.append((x, y, alt))
            x += hd.cellsize
            if max_alt is None or alt > max_alt:
                max_alt = alt
            if min_alt is None or alt < min_alt:
                min_alt = alt
        assert len(d_row)==hd.ncols
        yield d_row
        rows_processed += 1
        y -= hd.cellsize
    assert rows_processed==hd.nrows
    hd.max_value = max_alt
    hd.min_value = min_alt



def rebuild(gridfile, outfile):
    '''prepise puvodni ESRI grid na format vhodny k rychlemu hledani hodnoty pro zadane souradnice'''
    header = ESRIHeader.fromAsciiGridFile(gridfile)
    for row in parse_data(gridfile, header):
        for (x, y, alt) in row:
            outfile.write('%07.2f ' % alt)
    return header


