from PIL import Image
from tempfile import TemporaryFile
from cStringIO import StringIO
import sys, os


__all__ = [
    'ImageError',
    'loadImage', 'getMIME', 'getExt', 'getFileDesc', 'getStorageSize',
    'getImageData',
    'calcResize', 'calcAlign', 'calcCropForRatio',
    'ImageConverter', 'ThumbnailConverter', 'getImageInfo',
]

class ImageError(Exception):
    pass

ImageFormats = {
    'JPEG' : ('image/jpeg', 'jpg'),
    'GIF'  : ('image/gif', 'gif'),
    'PNG'  : ('image/png', 'png'),
    'TIFF' : ('image/tiff','tiff'),
    'BMP'  : ('image/bmp', 'bmp'),
    'PPM'  : ('image/x-portable-pixmap', 'ppm'),
    'XBM'  : ('image/x-xbitmap', 'xbm'),
    'XPM'  : ('image/x-xpixmap', 'xpm'),
}

def loadImage(image):
    if isinstance(image, Image.Image):
        pass
    elif isinstance(image, basestring):
        image = Image.open(image)
    elif hasattr(image, 'read') and hasattr(image, 'seek'):
        # StringIO, file or such
        image = Image.open(image)
    elif hasattr(image, 'disposition') and hasattr(image, 'type'):
        # cgi.FieldStorage (we don't want import cgi here)
        image = Image.open(image.file)
    else:
        raise ValueError("Unsupported image type %r" % image)
    return image

def getMIME(image):
    return ImageFormats[image.format][0]

def getExt(image):
    return ImageFormats[image.format][1]


def getFileDesc(image, forceNewFD=False):
    fp = image.fp
    if forceNewFD or fp is None:
        from tempfile import mkstemp
        (fd, nm) = mkstemp(suffix='.'+getExt(image))
        image.save(nm, image.format)
        fp = open(nm, 'rb')
        os.close(fd)
        os.unlink(nm)
    return fp

def getStorageSize(image, fp=None):
    if fp is None:
        fp = getFileDesc(image)
    o = fp.tell()
    fp.seek(0,2)
    s = fp.tell()
    fp.seek(o,0)
    return s

def getImageData(image):
    io = StringIO()
    #image.save(io, ImageFormats[image.format][1])
    image.save(io, image.format)
    return io.getvalue()






def calcResize((iWidth,iHeight), (maxWidth, maxHeight)):
    ratios = []
    if iWidth > maxWidth and maxWidth != 0:
        ratios.append(float(iWidth) / float(maxWidth))
    if iHeight > maxHeight and maxHeight != 0:
        ratios.append(float(iHeight) / float(maxHeight))
    if not ratios:
        # return original size
        return iWidth, iHeight
    else:
        # return new size that fits to limits
        ratio = max(ratios)
        iWidth = int(iWidth / ratio)
        iHeight = int(iHeight / ratio)
        return iWidth, iHeight

def calcAlign((iWidth,iHeight), (width, height), align='middle center'):
    alignMethod = {
        'left'  : ('h',lambda: 0),
        'center': ('h',lambda: max(iWidth-width,0)/2),
        'right' : ('h',lambda: max(iWidth-width,0)),
        'top'   : ('v',lambda: 0),
        'middle': ('v',lambda: max(iHeight-height,0)/2),
        'bottom': ('v',lambda: max(iHeight-height,0)),
    }
    am = dict([alignMethod[a] for a in align.split()])
    am.setdefault('h',alignMethod['center'][1])
    am.setdefault('v',alignMethod['middle'][1])
    hOff = am['h']()
    vOff = am['v']()
    width = min(iWidth, width)
    height = min(iHeight, height)
    return (hOff, vOff, hOff+width, vOff+height)

def calcCropForRatio((iWidth,iHeight), (rWidth, rHeight)):
    rRatio = float(rWidth) / float(rHeight)
    iRatio = float(iWidth) / float(iHeight)
    if rRatio >= iRatio:
        ratio = float(iWidth)/float(rWidth)
    else:
        ratio = float(iHeight)/float(rHeight)
    return int(rWidth*ratio), int(rHeight*ratio)



class ImageConverter:

    targetFormat = ('JPEG', 'PNG', 'GIF')

    maxSize = (sys.maxint, sys.maxint)

    ratio = 'keep' # keep/resize/crop/check/fixed

    align = 'middle center'

    def __init__(self, **kw):
        self.__dict__.update(kw)

    def calcCrop(self, image):
        iW,iH = image.size
        cW,cH = calcCropForRatio(image.size, self.maxSize)
        return (iW != cW or iH != cH), (cW, cH)

    def calcResize(self, image):
        iW,iH = image.size
        rW,rH = calcResize((iW, iH), self.maxSize)
        return (iW != rW or iH != rH), (rW,rH)

    def calcConvert(self, image, format):
        if format not in self.targetFormat:
            # XXX gif je lepsi konvertovat na png
            return True, self.targetFormat[0]
        else:
            return False, format

    def __call__(self, image, **kw):
        image = old = loadImage(image)
        format = old.format

        if self.ratio in ('crop', 'fixed'):
            crop, csize = self.calcCrop(image)
            if crop:
                image = image.crop(calcAlign(image.size, csize, self.align))
            if self.ratio == 'fixed':
                if image.size[0]!=self.maxSize[0] \
                        or image.size[1]!=self.maxSize[1]:
                    image = image.resize(self.maxSize, Image.ANTIALIAS)
            else:
                if image.size[0]>self.maxSize[0] \
                        or image.size[1]>self.maxSize[1]:
                    image = image.resize(self.maxSize, Image.ANTIALIAS)
        elif self.ratio == 'keep':
            resize, rsize = self.calcResize(image)
            if resize:
                image = image.resize(rsize, Image.ANTIALIAS)
        elif self.ratio == 'resize':
            if image.size[0]>self.maxSize[0] or image.size[1]>self.maxSize[1]:
                image = image.resize(self.maxSize, Image.ANTIALIAS)
        elif self.ratio == 'check':
            if image.size <> self.maxSize:
                raise ValueError('Invalid image size. Expected %d x %d)' % \
                                                                self.maxSize)
        else:
            raise ValueError('Unknown ratio method', self.ratio)

        convert, format = self.calcConvert(image, format)
        if convert or image is not old:
            #tf = TemporaryFile()
            tf = StringIO()
            if image.mode == 'P' and format == 'JPEG':
                image.convert("RGB").save(tf, format)
            else:
                image.save(tf, format)
            tf.seek(0,0)
            image = Image.open(tf)
        return image




class ThumbnailConverter(ImageConverter):

    targetFormat = ('PNG',)

    maxSize = (60,60)

    ratio = 'resize'



def getImageInfo(data):
    # From zope/app/file/image.py
    import struct

    data = str(data)
    size = len(data)
    height = -1
    width = -1
    content_type = ''

    # handle GIFs
    if (size >= 10) and data[:6] in ('GIF87a', 'GIF89a'):
        # Check to see if content_type is correct
        content_type = 'image/gif'
        w, h = struct.unpack("<HH", data[6:10])
        width = int(w)
        height = int(h)

    # See PNG 2. Edition spec (http://www.w3.org/TR/PNG/)
    # Bytes 0-7 are below, 4-byte chunk length, then 'IHDR'
    # and finally the 4-byte width, height
    elif ((size >= 24) and data.startswith('\211PNG\r\n\032\n')
          and (data[12:16] == 'IHDR')):
        content_type = 'image/png'
        w, h = struct.unpack(">LL", data[16:24])
        width = int(w)
        height = int(h)

    # Maybe this is for an older PNG version.
    elif (size >= 16) and data.startswith('\211PNG\r\n\032\n'):
        # Check to see if we have the right content type
        content_type = 'image/png'
        w, h = struct.unpack(">LL", data[8:16])
        width = int(w)
        height = int(h)

    # handle JPEGs
    elif (size >= 2) and data.startswith('\377\330'):
        content_type = 'image/jpeg'
        jpeg = StringIO(data)
        jpeg.read(2)
        b = jpeg.read(1)
        try:
            w = -1
            h = -1
            while (b and ord(b) != 0xDA):
                while (ord(b) != 0xFF): b = jpeg.read(1)
                while (ord(b) == 0xFF): b = jpeg.read(1)
                if (ord(b) >= 0xC0 and ord(b) <= 0xC3):
                    jpeg.read(3)
                    h, w = struct.unpack(">HH", jpeg.read(4))
                    break
                else:
                    jpeg.read(int(struct.unpack(">H", jpeg.read(2))[0])-2)
                b = jpeg.read(1)
            width = int(w)
            height = int(h)
        except struct.error:
            pass
        except ValueError:
            pass

    return content_type, width, height
