__all__ = ['gen_salt', 'parse_password', 'crypt_password', 'check_password']



def gen_salt(len=None, min=6, max=12, valid_letters=None):
    import random, string
    if valid_letters is None:
        valid_letters = string.ascii_letters + string.digits
    if len is None:
        len = random.randint(min, max)
    return ''.join([random.choice(valid_letters) for i in range(len)])


def nthash(password):
    # V baliku py-smbpasswd je C implementace
    # http://barryp.org/software/py-smbpasswd/
    import hashlib
    # potrebuji unicode
    if not isinstance(password, unicode):
        password = password.decode('utf-8')
    # v originale je prevod do UCS2, ale tohle je myslim ekvivalentni
    password = password.encode('utf_16_le')
    md4 = hashlib.new('md4')
    md4.update(password)
    return md4.hexdigest().upper()



def crypt_PLAIN(plain, salt=None):
    return plain

def crypt_CRYPT(plain, salt=None):
    import crypt
    if salt is None:
        import random
        salt = gen_salt()
    return crypt.crypt(plain, '$1$%s$' % salt)

def crypt_SHA(plain, salt=None):
    from hashlib import sha1
    return sha1(plain).digest().encode('base64').strip()

def crypt_SSHA(plain, salt=None):
    from hashlib import sha1
    if salt is None:
        salt = gen_salt()
    return (sha1(plain+salt).digest()+salt).encode('base64').strip()

def crypt_MD5(plain, salt=None):
    from hashlib import md5
    return md5(plain).digest().encode('base64').strip()

def crypt_SMD5(plain, salt=None):
    from hashlib import md5
    if salt is None:
        salt = gen_salt()
    return (md5(plain+salt).digest()+salt).encode('base64').strip()

def crypt_NTHASH(plain, salt=None):
    return nthash(plain)


def check_PLAIN(password, plain):
    return password == plain

def check_CRYPT(password, plain):
    import crypt
    return crypt.crypt(plain, password) == password

def check_SHA(password, plain):
    from hashlib import sha1
    return sha1(plain).digest().encode('base64').strip() == password

def check_SSHA(password, plain):
    from hashlib import sha1
    salt = password.decode('base64')[20:]
    return (sha1(plain+salt).digest()+salt).encode('base64').strip() == password

def check_MD5(password, plain):
    from hashlib import md5
    return md5(plain).digest().encode('base64').strip() == password

def check_SMD5(password, plain):
    from hashlib import md5
    salt = password.decode('base64')[16:]
    return (md5(plain+salt).digest()+salt).encode('base64').strip() == password

def check_NTHASH(password, plain):
    return nthash(plain) == password



REGISTRY = dict(
    PLAIN = (crypt_PLAIN, check_PLAIN),
    CLEAR = (crypt_PLAIN, check_PLAIN),
    CLEARTEXT = (crypt_PLAIN, check_PLAIN),
    CRYPT = (crypt_CRYPT, check_CRYPT),
    SHA = (crypt_SHA, check_SHA),
    SSHA = (crypt_SSHA, check_SSHA),
    MD5 = (crypt_MD5, check_MD5),
    SMD5 = (crypt_SMD5, check_SMD5),
    NTHASH = (crypt_NTHASH, check_NTHASH),
)

def parse_password(password):
    if not password:
        return None, None
    if password[:1] != '{':
        raise ValueError('Invalid password format', password)
    parts = password[1:].split('}', 1)
    if len(parts) != 2:
        raise ValueError('Invalid password format', password)
    method, passwd = parts
    if method not in REGISTRY:
        raise ValueError('Unknown password type', method)
    return method, passwd

def crypt_password(plain, method='SSHA', salt=None):
    if not plain:
        return None
    password = REGISTRY[method][0](plain, salt)
    return '{%s}%s' % (method, password)

def check_password(password, plain):
    if not password:
        return False
    method, crypted = parse_password(password)
    if not crypted:
        return False
    return REGISTRY[method][1](crypted, plain)

USAGE = '''

%%prog crypt plain [method [salt]] 
%%prog check plain crypted

Available encryption methods:
%s
''' % '\n'.join(' * %s' % m for m in sorted(REGISTRY))

def main():
    import sys, optparse
    parser = optparse.OptionParser(usage=USAGE)
    parser.add_option('-q', '--quiet', action='store_true', default=False)
    opts, av = parser.parse_args()
    ac = len(av)
    if not av:
        parser.error('no args')
    if 'crypt'.startswith(av[0]):
        if ac == 2:
            plain = av[1]
            method = 'CRYPT'
            salt = None
        elif ac == 3:
            plain = av[1]
            method = av[2]
            salt = None
        elif ac == 4:
            plain = av[1]
            method = av[2]
            salt = av[3]
        else:
            parser.error('args: crypt plain [method [salt]]')
        method = method.upper()
        if method not in REGISTRY:
            parser.error('unknown method %r' % method)
        enc = crypt_password(plain, method, salt)
        print enc
    elif 'check'.startswith(av[0]):
        if ac == 3:
            plain = av[1]
            crypted = av[2]
            res = check_password(av[2], plain)
            if res:
                if not opts.quiet:
                    print 'ok'
                sys.exit(0)
            else:
                if not opts.quiet:
                    print 'invalid'
                sys.exit(1)
        else:
            parser.error('args: check plain crypted')
    else:
        parser.error('unknown command')
    return


if __name__ == '__main__':
    main()
