# -*- coding: utf-8 -*-

# See also
#
# Debugging a running python process ...
# http://code.activestate.com/recipes/576515/
#
# pytelnetd
# http://www.geocities.com/chrisliechti/en/python/index.html
#
# Getting stack trace from a running Python application
# http://stackoverflow.com/questions/132058/getting-stack-trace-from-a-running-python-application
#

# Summer of Code
# http://svn.python.org/projects/sandbox/trunk/pdb/

import code, sys, os, signal, traceback

CONSOLE_SIGNAL = signal.SIGUSR1
SOCKET_PATH = '/tmp/remote_console.%d'


def socket_path(pid):
    return SOCKET_PATH % int(pid)


# requests
CMD_QUIT     = 'Q'
CMD_SINGLE   = 'S'
CMD_COMPLETE = 'C'
CMD_FEATURES = 'F'

# replies
REP_HELO     = 'h'
REP_STDOUT   = 'o'
REP_STDERR   = 'e'
REP_COMPLETE = 'c'


def parse_frame(buff):
    import struct
    l = len(buff)
    if l < 5:
        return None, None
    size = struct.unpack('!i', buff[1:5])[0]
    if l < size+5:
        return None, None
    return buff[0], buff[5:5+size]
    
def make_frame(cmd, data):
    import struct
    return cmd + struct.pack('!i', len(data)) + data





def setup_remote_console():
    import signal
    signal.signal(CONSOLE_SIGNAL, remote_interpret_handler)


def remote_interpret_handler(sig, frame):
    import socket
    path = socket_path(os.getpid())
    sck = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
    sck.bind(path)
    sck.listen(1)
    con, addr = sck.accept()
    sck.close()
    try:
        remote_interpret(con)
    finally:
        try:
            con.close()
        except:
            pass
        try:
            os.unlink(path)
        except:
            pass

def utils_tb():
    stack = traceback.extract_stack(limit=100)[:-3]
    return ''.join(traceback.format_list(stack))


def remote_interpret(con):
    from StringIO import StringIO

    locals = {
        "__name__"    : "remote_console",
        "__doc__"     : None,
        "traceback"   : traceback,
        "tb"          : utils_tb,
        "sys"         : sys,
        "os"          : os,
    }

    # code completer
    matches = []
    def complete(text):
        import keyword, __builtin__, re
        def get_class_members(klass):
            ret = dir(klass)
            if hasattr(klass,'__bases__'):
                for base in klass.__bases__:
                    ret = ret + get_class_members(base)
            return ret
        if "." in text:
            m = re.match(r"(\w+(\.\w+)*)\.(\w*)", text)
            if not m:
                return []
            expr, attr = m.group(1, 3)
            object = eval(expr, locals)
            words = dir(object)
            if hasattr(object,'__class__'):
                words.append('__class__')
                words = words + get_class_members(object.__class__)
            matches = []
            n = len(attr)
            for word in words:
                if word[:n] == attr and word != "__builtins__":
                    matches.append("%s.%s" % (expr, word))
            return matches
        else:
            matches = []
            n = len(text)
            for list in [keyword.kwlist, __builtin__.__dict__, locals]:
                for word in list:
                    if word[:n] == text and word != "__builtins__":
                        matches.append(word)
            return matches

    # send hello
    hello = [
        "Remote Python %s on %s\n\n" % (sys.version, sys.platform),
        "Remote traceback:\n%s\n" % utils_tb(),
        "Predefined names:\n",
        " * os - module\n",
        " * sys - module\n",
        " * traceback - module\n",
        " * tb() - returns formated current traceback (use ``print tb()'')\n"
    ]
    con.send(make_frame(REP_HELO, ''.join(hello)))

    # process commands
    buff = ''
    while True:
        data = con.recv(1024)
        if not data:
            break
        buff += data
        cmd, arg = parse_frame(buff)
        if cmd is None:
            continue
        buff = ''
        #print 'CMD', (cmd, arg)

        if cmd == CMD_COMPLETE:
            names = complete(arg)
            con.send(make_frame(REP_COMPLETE, ','.join(names)))

        elif cmd == CMD_SINGLE:
            out = StringIO()
            err = StringIO()
            oldout, sys.stdout = sys.stdout, out
            olderr, sys.stderr = sys.stderr, err
            oldin,  sys.stdin  = sys.stdin,  None
            try:
                try:
                    code = compile(arg, '<remote_console>', 'single')
                except:
                    tb = traceback.format_exception(*sys.exc_info())
                    con.send(make_frame(REP_STDERR, tb))
                else:
                    try:
                        exec code in locals
                    except:
                        tb = traceback.format_exception(*sys.exc_info())
                        con.send(make_frame(REP_STDERR, ''.join(tb)))
                    else:
                        con.send(make_frame(REP_STDOUT, out.getvalue()))
            finally:
                sys.stdout = oldout
                sys.stderr = olderr
                sys.stdin  = oldin
        elif cmd == CMD_QUIT:
            break
        else:
            break

    
class RemoteConsole(code.InteractiveConsole):

    def __init__(self, pid):
        import socket, readline, codeop, time

        readline.set_completer(self.complete)
        readline.parse_and_bind("tab: complete")

        os.kill(pid, CONSOLE_SIGNAL)

        self.filename = '<remote console>'
        self.resetbuffer()
        self.compile = codeop.CommandCompiler()

        path = socket_path(pid)
        sys.stdout.write('connecting...')
        sys.stdout.flush()
        while True:
            if os.path.exists(path):
                break
            sys.stdout.write('.')
            sys.stdout.flush()
            time.sleep(0.1)
        self.con = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        self.con.connect(path)
        print 'connected!'
        print
        cmd, arg = self.receive()
        assert cmd == REP_HELO
        print arg.rstrip()

    def runsource(self, source, filename="<input>", symbol="single"):
        try:
            code = self.compile(source, filename, symbol)
        except (OverflowError, SyntaxError, ValueError):
            # Case 1
            self.showsyntaxerror(filename)
            return False
        if code is None:
            # Case 2
            return True
        # Case 3
        self.runcode(code, source)
        return False

    def receive(self):
        buff = ''
        while True:
            data = self.con.recv(1024)
            if not data:
                return None, None
            buff += data
            cmd, arg = parse_frame(buff)
            if cmd is not None:
                return cmd, arg
        
    def runcode(self, code, source):
        self.con.send(make_frame(CMD_SINGLE, source))
        cmd, arg = self.receive()
        if cmd == REP_STDOUT:
            sys.stdout.write(arg)
            sys.stdout.flush()
        elif cmd == REP_STDERR:
            sys.stderr.write(arg)
            sys.stderr.flush()
        elif cmd is None:
            print 'CLOSED'
        else:
            print 'UNKNOWN REPLY', (cmd, arg)

    def complete(self, text, state):
        # XXX optimalizace - pokud je predchozi text prefixem aktualniho
        #     a souhlasi pocet tecek, tak nemusim zjistovat jmena na druhe
        #     strane, ale muzu si je spocitat tady
        if state == 0:
            self.con.send(make_frame(CMD_COMPLETE, text))
            cmd, arg = self.receive()
            assert cmd == REP_COMPLETE
            if not arg:
                self.names = []
            else:
                self.names = arg.split(',')
        try:
            return self.names[state]
        except IndexError:
            return None

    def __del__(self):
        try:
            self.con.close()
        except:
            pass

def test_process():
    print os.getpid()
    import time, itertools
    setup_remote_console()
    pid = os.getpid()
    def a():
        counter = itertools.count().next
        while True:
            c = counter()
            #print pid, counter()
            #time.sleep(1)
        return None
    def b():
        return a()
    def c():
        return b()
    return c()


def main():
    pid = int(sys.argv[1])
    console = RemoteConsole(pid)
    console.interact(banner='')

if __name__ == '__main__':
    main()
