import unittest
from lib4.cgi import *
from os.path import abspath, split, join
import cgi

THISDIR = abspath(join(split(__file__)[0]))
IMGDIR = join(THISDIR, 'imgs')
HTMDIR = join(THISDIR, 'html')

class TestTestEnviron(unittest.TestCase):

    def test10_GetWithoutParams(self):
        e,i = test_environ()
        self.assertEqual(e['REQUEST_METHOD'], 'GET')
        fs = cgi.FieldStorage(fp=i, environ=e)
        self.assertEqual(len(fs.keys()), 0)
        self.failIf(e['QUERY_STRING'])

    def test11_GetWithParams(self):
        e,i = test_environ(get={'a':1,'b':'2+=; %2'})
        self.assertEqual(e['REQUEST_METHOD'], 'GET')
        self.failIf('CONTENT_TYPE' in e)
        fs = cgi.FieldStorage(fp=i, environ=e)
        self.assertEqual(len(fs.keys()), 2)
        self.assertEqual(fs['a'].value, '1')
        self.assertEqual(fs['b'].value, '2+=; %2')
        self.failUnless(e['QUERY_STRING'])

    def test12_GetWithQueryString(self):
        e,i = test_environ('?a=1&b=2')
        self.assertEqual(e['REQUEST_METHOD'], 'GET')
        self.failIf('CONTENT_TYPE' in e)
        fs = cgi.FieldStorage(fp=i, environ=e)
        self.assertEqual(len(fs.keys()), 2)
        self.assertEqual(fs['a'].value, '1')
        self.assertEqual(fs['b'].value, '2')
        self.failUnless(e['QUERY_STRING'])

    def test13_GetWithParamsOnStdin(self):
        e,i = test_environ(
            env={'REQUEST_METHOD':'GET'},
            stdin=encode_qs({'a':1,'b':'2+=; %2'})
        )
        self.assertEqual(e['REQUEST_METHOD'], 'GET')
        self.failIf('CONTENT_TYPE' in e)
        fs = cgi.FieldStorage(fp=i, environ=e)
        self.assertEqual(len(fs.keys()), 2)
        self.assertEqual(fs['a'].value, '1')
        self.assertEqual(fs['b'].value, '2+=; %2')
        self.failUnless(e['QUERY_STRING'])

    def test20_PostWithoutParams(self):
        e,i = test_environ(env={'REQUEST_METHOD':'POST'})
        self.assertEqual(e['REQUEST_METHOD'], 'POST')
        self.assertEqual(e['CONTENT_TYPE'], 'application/x-www-form-urlencoded')
        self.assertEqual(e['CONTENT_LENGTH'], '0')
        self.failIf(e['QUERY_STRING'])
        fs = cgi.FieldStorage(fp=i, environ=e)
        self.assertEqual(len(fs.keys()), 0)

    def test21_PostWithParams(self):
        e,i = test_environ(post={'a':1,'b':'2+=; %2'})
        self.assertEqual(e['REQUEST_METHOD'], 'POST')
        self.assertEqual(e['CONTENT_TYPE'], 'application/x-www-form-urlencoded')
        self.assertEqual(e['CONTENT_LENGTH'], str(len(i.getvalue())))
        self.failIf(e['QUERY_STRING'])
        fs = cgi.FieldStorage(fp=i, environ=e)
        self.assertEqual(len(fs.keys()), 2)
        self.assertEqual(fs['a'].value, '1')
        self.assertEqual(fs['b'].value, '2+=; %2')

    def test23_PostWithParamsOnStdin(self):
        e,i = test_environ(stdin=encode_qs({'a':1,'b':'2+=; %2'}))
        self.assertEqual(e['REQUEST_METHOD'], 'POST')
        self.assertEqual(e['CONTENT_TYPE'], 'application/x-www-form-urlencoded')
        fs = cgi.FieldStorage(fp=i, environ=e)
        self.assertEqual(len(fs.keys()), 2)
        self.assertEqual(fs['a'].value, '1')
        self.assertEqual(fs['b'].value, '2+=; %2')


    def test30_MultipartWithParams(self):
        e,i = test_environ(
            post={'a':1,'b':'2+=; %2'},
            env={'CONTENT_TYPE':'multipart/form-data'},
        )
        self.assertEqual(e['REQUEST_METHOD'], 'POST')
        assert e['CONTENT_TYPE'].startswith('multipart/form-data; boundary=')
        self.assertEqual(e['CONTENT_LENGTH'], str(len(i.getvalue())))
        self.failIf(e['QUERY_STRING'])
        fs = cgi.FieldStorage(fp=i, environ=e)
        self.assertEqual(len(fs.keys()), 2)
        self.assertEqual(fs['a'].value, '1')
        self.assertEqual(fs['b'].value, '2+=; %2')

    def test31_MultipartWithMultiParams(self):
        e,i = test_environ(
            post={'a':[1,3],'b':(2,4)},
            env={'CONTENT_TYPE':'multipart/form-data'},
        )
        self.assertEqual(e['REQUEST_METHOD'], 'POST')
        assert e['CONTENT_TYPE'].startswith('multipart/form-data; boundary=')
        self.assertEqual(e['CONTENT_LENGTH'], str(len(i.getvalue())))
        self.failIf(e['QUERY_STRING'])
        fs = cgi.FieldStorage(fp=i, environ=e)
        self.assertEqual(len(fs.keys()), 2)
        self.assertEqual(fs.getlist('a'), ['1','3'])
        self.assertEqual(fs.getlist('b'), ['2','4'])

    def test31_MultipartWithDataOnStdin(self):
        d,c = encode_multipart({'a':1,'b':'2+=; %2'})
        e,i = test_environ(
            env={'REQUEST_METHOD':'POST','CONTENT_TYPE':'multipart/form-data'},
            stdin=d,
        )
        self.assertEqual(e['REQUEST_METHOD'], 'POST')
        self.assertEqual(e['CONTENT_TYPE'], c)
        self.assertEqual(e['CONTENT_LENGTH'], str(len(i.getvalue())))
        self.failIf(e['QUERY_STRING'])
        fs = cgi.FieldStorage(fp=i, environ=e)
        self.assertEqual(len(fs.keys()), 2)
        self.assertEqual(fs['a'].value, '1')
        self.assertEqual(fs['b'].value, '2+=; %2')

    def test35_MultipartWithTwoFiles(self):
        jpeg = join(IMGDIR,'p.jpeg')
        html = join(HTMDIR,'nonsense.html')
        e,i = test_environ(
            post={'a':1, 'f':TestUpload(jpeg), 'h':TestUpload(html)},
        )
        self.assertEqual(e['REQUEST_METHOD'], 'POST')
        assert e['CONTENT_TYPE'].startswith('multipart/form-data; boundary=')
        self.assertEqual(e['CONTENT_LENGTH'], str(len(i.getvalue())))
        self.failIf(e['QUERY_STRING'])
        fs = cgi.FieldStorage(fp=i, environ=e)
        self.assertEqual(len(fs.keys()), 3)
        self.assertEqual(fs['a'].value, '1')
        f = fs['f']
        self.assertEqual(f.type, 'image/jpeg')
        self.assertEqual(f.filename, 'p.jpeg')
        self.assertEqual(f.name, 'f')
        self.assertEqual(fs['f'].value, open(jpeg, 'rb').read())
        h = fs['h']
        self.assertEqual(h.type, 'text/html')
        self.assertEqual(h.filename, 'nonsense.html')
        self.assertEqual(h.name, 'h')
        self.assertEqual(fs['h'].value, open(html, 'rb').read())

    def test40_Auth(self):
        e,_ = test_environ(auth=('joe','pwd'))
        self.assertEqual(e['AUTH_TYPE'], 'Basic')
        u,p = parse_auth(e['HTTP_AUTHORIZATION'])
        self.assertEqual(u, 'joe')
        self.assertEqual(p, 'pwd')

    def test50_UploadWithoutSize(self):
        u = join(HTMDIR,'nonsense.html')
        e,i = test_environ(post={'h':TestUpload(u, size=None)})
        self.failIf('Content-length' in i.getvalue())
        fs = cgi.FieldStorage(fp=i, environ=e)
        h = fs['h']
        self.assertEqual(h.type, 'text/html')
        self.assertEqual(h.filename, 'nonsense.html')
        self.assertEqual(h.name, 'h')
        self.assertEqual(fs['h'].value, open(u, 'rb').read())
