import unittest
from lib4.procutils import *
import os, sys, signal, time



class TestForkCall(unittest.TestCase):

    def test10_CallWithoutError(self):
        fc = ForkCall(lambda: 42)
        self.assertEqual(os.kill(fc.pid, 0), None)
        res = fc()
        self.assertRaises(OSError, os.kill, fc.pid, 0)
        self.assertEqual(res, 42)

    def test11_CallWithError(self):
        def mkError():
            raise Exception()
        fc = ForkCall(mkError)
        self.assertRaises(ForkCallError, fc)
        self.assertRaises(OSError, os.kill, fc.pid, 0)

    def test12_LargeResult(self):
        fc = ForkCall(lambda: 'a' * 1024 * 128)
        self.assertEqual(os.kill(fc.pid, 0), None)
        self.assertEqual(os.kill(fc.pid, 0), None)
        res = fc()
        self.assertEqual(res, 'a' * 1024 * 128)
        self.assertRaises(OSError, os.kill, fc.pid, 0)

    def test13_ExitedChild(self):
        def mkError():
            os._exit(1)
        fc = ForkCall(mkError)
        self.assertRaises(ForkCallError, fc)
        self.assertRaises(OSError, os.kill, fc.pid, 0)


def mkTemp(pid):
    return '/tmp/test_procutils_%d.lock' % pid




class TestProcessInstanceLock(unittest.TestCase):

    def setUp(self):
        self.forUnlink = []

    def tearDown(self):
        for f in self.forUnlink:
            try:
                os.unlink(f)
            except:
                pass

    def test10_NonconflictingBlockingLock(self):
        pid=os.getpid(); file=mkTemp(pid)
        self.forUnlink.append(file)
        lck = ProcessInstanceLock(file, pid)
        self.assertEqual(lck.acquire(True), True)
        lck.release()
        self.failUnless(os.path.isfile, file)

    def test11_NonconflictingNonblockingLock(self):
        pid=os.getpid(); file=mkTemp(pid)
        self.forUnlink.append(file)
        lck = ProcessInstanceLock(file, pid)
        self.assertEqual(lck.acquire(False), True)
        lck.release()
        self.failUnless(os.path.isfile, file)

    def test20_MultipleBlockingAcquire(self):
        pid=os.getpid(); file=mkTemp(pid)
        self.forUnlink.append(file)
        lck = ProcessInstanceLock(file, pid)
        self.assertEqual(lck.acquire(False), True)
        self.assertRaises(ProcessInstanceLockError, lck.acquire, True)
        lck.release()

    def test21_MultipleNonblockingAcquire(self):
        pid=os.getpid(); file=mkTemp(pid)
        self.forUnlink.append(file)
        lck = ProcessInstanceLock(file, pid)
        self.assertEqual(lck.acquire(False), True)
        self.assertRaises(ProcessInstanceLockError, lck.acquire, False)
        lck.release()

    def test22_MultipleRelease(self):
        pid=os.getpid(); file=mkTemp(pid)
        self.forUnlink.append(file)
        lck = ProcessInstanceLock(file, pid)
        self.assertEqual(lck.acquire(False), True)
        lck.release()
        self.assertRaises(ProcessInstanceLockError, lck.release)

    def test23_LockingExistingFile(self):
        pid=os.getpid(); file=mkTemp(pid)
        self.forUnlink.append(file)
        open(file, 'w').write('42\n')
        lck = ProcessInstanceLock(file, pid)
        self.assertEqual(lck.acquire(False), True)
        lck.release()

    def test30_BlockingConcurrentLock(self):
        pid = os.getpid();
        file = mkTemp(pid)
        self.forUnlink.append(file)

        # signal handler with registration
        received = {}
        def ignore(*a):
            received[os.getpid()] = True
        signal.signal(signal.SIGHUP, ignore)

        child = os.fork()
        if child == 0:
            # children
            lck = ProcessInstanceLock(file)
            lck.acquire(True)
            # notify parent that lock is acquired
            os.kill(pid, signal.SIGHUP)
            # wait for signal to release lock if signal was not received
            # already
            if not received.get(os.getpid()):
                try: time.sleep(30)
                except: pass
            lck.release()
            os._exit(0)

        else:
            lck = ProcessInstanceLock(file)
            # wait for signal that concurrent lock is acquired
            try: time.sleep(30)
            except: pass
            # nonblocking lock should return False
            self.assertEqual(lck.acquire(False), False)
            # send signal to child and wait for its completion
            os.kill(child, signal.SIGHUP)
            os.waitpid(child, 0)
            # nonblocking lock should return True
            self.assertEqual(lck.acquire(False), True)
