import unittest
from lib4.properties import *
from lib4.testing import Record



class TestAttrProxy(unittest.TestCase):

    # Test is imported from wm4.utils.tests.test_binding

    class Rec(Record):
        ap1 = AttrProxy('a')
        ap2 = AttrProxy('r/a')
        pp1 = AttrProxy('ap1')
        fp2 = AttrProxy('f/foo')

    class Foo(Record):
        def foo(self):
            return self

    def test11_Get(self):
        r = self.Rec()
        self.assertRaises(AttributeError, getattr, r, 'ap1')
        r.a = 1
        self.assertEqual(r.ap1, 1)

    def test14_SetNonexisting(self):
        r = self.Rec()
        r.ap1 = 1
        self.assertEqual(r.a, 1)
        self.assertEqual(r.ap1, 1)
        self.failIf('ap1' in r.__dict__)
        self.failUnless('a' in r.__dict__)

    def test15_SetExisting(self):
        r = self.Rec(a=0)
        r.ap1 = 1
        self.assertEqual(r.a, 1)
        self.assertEqual(r.ap1, 1)
        self.failIf('ap1' in r.__dict__)
        self.failUnless('a' in r.__dict__)

    def test17_DelExisting(self):
        r = self.Rec(a=1)
        del r.ap1
        self.failIf('a' in r.__dict__)
        self.failIf('ap1' in r.__dict__)
        self.assertRaises(AttributeError, delattr, r, 'ap1')

    def test21_Get(self):
        r = self.Rec()
        self.assertRaises(AttributeError, getattr, r, 'ap2')
        r.r = Record(a=1)
        self.assertEqual(r.ap2, 1)
        r = self.Rec(r=Record())
        self.assertRaises(AttributeError, getattr, r, 'ap2')
        r.r.a = 1
        self.assertEqual(r.ap2, 1)

    def test24_SetNonexisting(self):
        r = self.Rec()
        self.assertRaises(AttributeError, setattr, r, 'ap2', 1)
        r = self.Rec(r=Record())
        r.ap2 = 1
        self.assertEqual(r.r.a, 1)
        self.assertEqual(r.ap2, 1)
        self.failIf('ap2' in r.__dict__)
        self.failUnless('a' in r.r.__dict__)

    def test25_SetExisting(self):
        r = self.Rec(r=Record(a=0))
        r.ap2 = 1
        self.assertEqual(r.r.a, 1)
        self.assertEqual(r.ap2, 1)
        self.failIf('ap2' in r.__dict__)
        self.failUnless('a' in r.r.__dict__)

    def test27_DelExisting(self):
        r = self.Rec(r=Record(a=1))
        del r.ap2
        self.failIf('a' in r.r.__dict__)
        self.failIf('ap2' in r.__dict__)
        self.assertRaises(AttributeError, delattr, r, 'ap2')

    def test30_ProxyToProxyGet(self):
        r = self.Rec(a=1)
        self.assertEqual(r.pp1, 1)
        r = self.Rec()
        self.assertRaises(AttributeError, getattr, r, 'pp1')

    def test31_ProxyToProxySet(self):
        r = self.Rec(a=0)
        r.pp1 = 1
        self.assertEqual(r.pp1, 1)
        r = self.Rec()
        r.pp1 = 1
        self.assertEqual(r.pp1, 1)

    def test32_ProxyToProxyDel(self):
        r = self.Rec(a=0)
        del r.pp1
        self.failIf('a' in r.__dict__)
        self.assertRaises(AttributeError, delattr, r, 'pp1')

    def test40_ProxyToMethod(self):
        r = self.Rec(f=self.Foo())
        self.assertEqual(r.fp2, r.f.foo)
        self.assertEqual(r.fp2(), r.f.foo())

    def test41_ProxyToMethodSet(self):
        r = self.Rec(f=self.Foo())
        r.fp2 = lambda:42
        self.assertEqual(r.fp2(), 42)

    def test42_ProxyToMethodDel(self):
        r = self.Rec(f=self.Foo())
        self.assertRaises(AttributeError, delattr, r, 'fp2')









class TestLateBindingProperty(unittest.TestCase):

    def test10_SimpleGetter(self):
        class C(object):
            def getx(self): return 42
            x = LateBindingProperty(getx)
        self.assertEqual(C().x, 42)

    def test11_ReplacingGetterInSubclass(self):
        class C(object):
            def getx(self): return 42
            x = LateBindingProperty(getx)
        class D(C):
            def getx(self): return 43
        self.assertEqual(D().x, 43)

    def test20_SimpleSetter(self):
        class C(object):
            def setx(self, val): self.__dict__['_x'] = val
            x = LateBindingProperty(None, setx)
        c = C()
        c.x = 42
        self.assertEqual(c.__dict__['_x'], 42)

    def test21_ReplacingSetterInSubclass(self):
        class C(object):
            def setx(self, val): self.__dict__['_x'] = val
            x = LateBindingProperty(None, setx)
        class D(C):
            def setx(self, val): self.__dict__['_x'] = val * 2
        d = D()
        d.x = 42
        self.assertEqual(d.__dict__['_x'], 84)
