import unittest
from lib4.runtime import *


class TestCombineClasses(unittest.TestCase):

    def test20_DisjuctMetaclasses(self):
        class M1(type): pass
        class M2(type): pass
        class C1(object): __metaclass__ = M1
        class C2(object): __metaclass__ = M2

        C = superclass(bases=(C1, C2))
        self.assertEqual(C.__bases__, (C1, C2))
        self.assertEqual(type(C).__bases__, (M1, M2))

        C = superclass(bases=(C2, C1))
        self.assertEqual(C.__bases__, (C2, C1))
        self.assertEqual(type(C).__bases__, (M2, M1))

    def test21_ConjuctMetaclasses(self):
        class M1(type): pass
        class M2(M1): pass
        class C1(object): __metaclass__ = M1
        class C2(object): __metaclass__ = M2

        C = superclass(bases=(C1, C2))
        self.assertEqual(C.__bases__, (C1, C2))
        self.assertEqual(type(C), M2)

        C = superclass(bases=(C2, C1))
        self.assertEqual(C.__bases__, (C2, C1))
        self.assertEqual(type(C), M2)



class TestSignature(unittest.TestCase):

    # args, varargs, varkw, defaults

    def test01_Function(self):
        def f(): pass
        self.assertEqual(signature(f), ([], None, None, {}))

    def test02_Function(self):
        def f(a): pass
        self.assertEqual(signature(f), (['a'], None, None, {}))

    def test03_Function(self):
        def f(*v): pass
        self.assertEqual(signature(f), ([], 'v', None, {}))

    def test04_Function(self):
        def f(a, b=1, c=2, *v, **k): pass
        self.assertEqual(signature(f), (['a','b','c'],'v','k',dict(b=1,c=2)))

    def test10_BoundMethod(self):
        class X:
            def f(self): pass
        self.assertEqual(signature(X().f), ([], None, None, {}))

    def test11_BoundMethod(self):
        class X:
            def f(self, a, b=1, c=2, *v, **k): pass
        self.assertEqual(signature(X().f),(['a','b','c'],'v','k',dict(b=1,c=2)))

    def test15_UnboundMethod(self):
        class X:
            def f(self): pass
        self.assertEqual(signature(X.f), (['self'], None, None, {}))

    def test16_UnboundMethod(self):
        class X:
            def f(self, a, b=1, *v, **k): pass
        self.assertEqual(signature(X.f), (['self','a','b'], 'v', 'k',dict(b=1)))

    def test20_Constructor(self):
        class X:
            def __init__(self, a, b=1, *v, **k):
                pass
        self.assertEqual(signature(X), (['a','b'], 'v', 'k', dict(b=1)))
        self.assertRaises(TypeError, signature, X(1))

    def test30_CallableObject(self):
        class X:
            def __init__(self, b): pass
            def __call__(self, a): pass
        self.assertEqual(signature(X(1)), (['a'], None, None, {}))
        self.assertEqual(signature(X), (['b'], None, None, {}))

    def test31_CallableObject(self):
        class X:
            def __call__(self, a): pass
        self.assertEqual(signature(X()), (['a'], None, None, {}))
        self.assertEqual(signature(X), ([], None, None, {}))

    def test32_CallableObject(self):
        class Y:
            def __call__(self, a):
                pass
        class X:
            __call__ = Y()
        self.assertEqual(signature(X()), (['a'], None, None, {}))
        self.assertEqual(signature(X), ([], None, None, {}))

    def test32_CallableObject(self):
        class Y:
            def __call__(self, a):
                pass
        class X:
            def __init__(self):
                self.__call__ = Y()
        self.assertEqual(signature(X()), (['a'], None, None, {}))
        self.assertEqual(signature(X), ([], None, None, {}))

    def test40_CallableAttr(self):
        def f(a, b=1, c=2, *v, **k):
            pass
        class X:
            def __init__(self):
                self.f = f
        self.assertEqual(
            signature(X().f), (['a','b','c'], 'v', 'k',dict(b=1,c=2))
        )

    def test50_Descriptors(self):
        class X(object):
            def getf(obj):
                def f(a, b=1, c=2, *v, **k): pass
                return f
            f = property(getf)
        self.assertEqual(
            signature(X().f), (['a','b','c'], 'v', 'k',dict(b=1,c=2))
        )

    def test90_Noncallable(self):
        self.assertRaises(TypeError, signature, 1)
        self.assertRaises(TypeError, signature, 'a')
        class X(object):
            f = property(lambda s: 42)
        self.assertRaises(TypeError, signature, X().f)
        self.assertRaises(TypeError, signature, X.f)


class TestValuesForSignature(unittest.TestCase):

    def test01_Empty(self):
        av, kw = values_for_signature(([], None, None, {}))
        self.assertEqual(av, ())
        self.assertEqual(kw, {})

    def test02_Positional(self):
        av, kw = values_for_signature((['a','b'], None, None, {}))
        self.assertEqual(av, (None,None))
        self.assertEqual(kw, {})

    def test03_PositionalWithNamespace(self):
        av, kw = values_for_signature(
            (['a','b'], None, None, {}), namespace={'b':1}
        )
        self.assertEqual(av, (None,1))
        self.assertEqual(kw, {})

    def test04_PositionalWithDefault(self):
        av, kw = values_for_signature(
            (['a','b'], None, None, {}), namespace={'b':1}, default=42
        )
        self.assertEqual(av, (42,1))
        self.assertEqual(kw, {})

    def test05_PositionalWithDefaultDef(self):
        av, kw = values_for_signature(
            (['a','b'], None, None, dict(b=2)), default=42
        )
        self.assertEqual(av, (42,2))
        self.assertEqual(kw, {})

    def test06_PositionalWithDefaultDef(self):
        av, kw = values_for_signature(
            (['a','b'], None, None, dict(b=2)), namespace={'b':1}, default=42
        )
        self.assertEqual(av, (42,1))
        self.assertEqual(kw, {})

    def test10_VarArgs(self):
        av, kw = values_for_signature(
            (['a',], 'c', None, {}), namespace={'c':[2,3], 'a':1},
        )
        self.assertEqual(av, (1,2,3))
        self.assertEqual(kw, {})

    def test11_VarArgsWithFactory(self):
        av, kw = values_for_signature(
            (['a',], 'c', None, {}), namespace={'a':1},
            get_vararg = lambda a: [4,5],
        )
        self.assertEqual(av, (1,4,5))
        self.assertEqual(kw, {})

    def test12_VarArgsWithFactory(self):
        av, kw = values_for_signature(
            (['a',], 'c', None, {}), namespace={'c':[2,3], 'a':1},
            get_vararg = lambda a: [4,5],
        )
        self.assertEqual(av, (1,2,3))
        self.assertEqual(kw, {})

    def test20_VarKw(self):
        av, kw = values_for_signature(
            (['a',], None, 'k', {}), namespace={'k':{'x':4}, 'a':1},
        )
        self.assertEqual(av, (1,))
        self.assertEqual(kw, {'x':4})

    def test21_VarArgsWithFactory(self):
        av, kw = values_for_signature(
            (['a',], None, 'k', {}), namespace={'a':1},
            get_varkw = lambda a: {'y':5}
        )
        self.assertEqual(av, (1,))
        self.assertEqual(kw, {'y':5})

    def test22_VarArgsWithFactory(self):
        av, kw = values_for_signature(
            (['a',], None, 'k', {}), namespace={'k':{'x':4}, 'a':1},
            get_varkw = lambda a: {'y':5}
        )
        self.assertEqual(av, (1,))
        self.assertEqual(kw, {'x':4})



def f(a,b,c='d_c',d='d_d', *v, **k):
    return (a,b,c,d), v, k


class TestMApply(unittest.TestCase):

    def test01(self):
        p,v,k = mapply(f)
        self.assertEqual(p, (None,None,'d_c','d_d'))
        self.assertEqual(v, ())
        self.assertEqual(k, {})

    def test02(self):
        p,v,k = mapply(f,
            namespace={'a':'e_a', 'b':'e_b'},
            get_vararg = lambda a: ['v1','v2'],
            get_varkw = lambda a: dict(x=4, y=5),
        )
        self.assertEqual(p, ('e_a','e_b', 'd_c', 'd_d'))
        self.assertEqual(v, ('v1', 'v2'))
        self.assertEqual(k, dict(x=4, y=5))


