#-*- coding: iso-8859-2 -*-
import unittest
from lib4.text import *


class TestFTemplate(unittest.TestCase):

    templateFactory = fTemplate

    def checkResult(self, val, expected):
        self.assertEqual(val, str(expected))
        self.assert_(isinstance(val, str))

    def test1_Empty(self):
        t = self.templateFactory()
        self.checkResult(t.getStatic(), '')
        self.checkResult(t(), '')

    def test2_OnlyCallbacks(self):
        t = self.templateFactory(lambda:'a', lambda:'b')
        self.assertEqual(t.getStatic(), None)
        self.checkResult(t(), 'ab')

    def test3_OnlyTexts(self):
        t = self.templateFactory('a', 'b')
        self.checkResult(t.getStatic(), 'ab')
        self.checkResult(t(), 'ab')

    def test4_Combination(self):
        t = self.templateFactory('a', lambda x:'b', 'c', lambda x:'d'*x)
        self.assertEqual(t.getStatic(), None)
        self.checkResult(t(2), 'abcdd')

    def test5_AddingToStatic(self):
        t = self.templateFactory('a')
        self.checkResult(t.getStatic(), 'a')
        self.checkResult(t(2), 'a')
        t.write(lambda x:'b', 'c', lambda x:'d'*x)
        self.assertEqual(t.getStatic(), None)
        self.checkResult(t(2), 'abcdd')
        self.assertEqual(t.getStatic(), None)
        self.checkResult(t(3), 'abcddd')
        self.assertEqual(t.getStatic(), None)

    def test6_AddingFunAfterGen(self):
        t = self.templateFactory(lambda x:'a'*x)
        self.checkResult(t(1), 'a')
        t.write(lambda x:'b'*x)
        self.assertEqual(t.getStatic(), None)
        self.checkResult(t(2), 'aabb')


class TestUFTemplate(TestFTemplate):

    templateFactory = ufTemplate

    def checkResult(self, val, expected):
        self.assertEqual(val, unicode(expected))
        self.assert_(isinstance(val, unicode))


u2a = unicode2ascii
il2 = lambda s: unicode(s,'iso-8859-2')

class TestUnicode2ASCII(unittest.TestCase):

    def test10_ASCII(self):
        self.assertEqual(u2a(unicode('aBc01234','ascii')),'aBc01234')

    def test12_Latin2(self):
        self.assertEqual(u2a(il2('')),'aeiouy')

    def test13_Latin2Cap(self):
        self.assertEqual(u2a(il2('')),'AEIOUY')

    def test14_Latin2(self):
        self.assertEqual(u2a(il2('')),'cdelnrstuz')

    def test15_Latin2Cap(self):
        self.assertEqual(u2a(il2('̥ةٮ')),'CDELNRSTUZ')

    def test20_MyanmarIgnore(self):
        self.assertEqual(u2a(u'a\u1000b'),'ab')

    def test21_MyanmarReplace(self):
        self.assertEqual(u2a(u'a\u1000b','<?>'), 'a<?>b')

    def test22_MyanmarRaise(self):
        self.assertRaises(UnicodeEncodeError, u2a, u'a\u1000b', None)

    def test30_RomanNumber(self):
        self.assertEqual(u2a(u'a\u0049b'),'aIb')
        self.assertEqual(u2a(u'a\u2160b'),'aIb')



ip = lambda t: list(
        interpolation_parser(t,
            lambda s,l: type(t).lower(s),
            lambda s,l: type(t).upper(s),
    ))

class TestInterpolation(unittest.TestCase):

    def test01_NoInterpolation(self):
        self.assertEqual(ip('a'), ['a'])
        self.assertEqual(ip(il2('dm')), [il2('dm')])

    def test02_NoInterpolationEscape(self):
        self.assertEqual(ip('$$'), ['$'])
        self.assertEqual(ip('A$$'), ['a$'])
        self.assertEqual(ip('$$A'), ['$a'])
        self.assertEqual(ip('A$$A'), ['a$a'])

    def test03_SubstExpr(self):
        self.assertEqual(ip('${v.a.r}'), ['V.A.R'])
        self.assertEqual(ip('b${v.a.r}'), ['b', 'V.A.R'])
        self.assertEqual(ip('${v.a.r}e'), ['V.A.R', 'e'])
        self.assertEqual(ip('b${v.a.r}e'), ['b', 'V.A.R', 'e'])

    def test04_ShortSubstExpr(self):
        self.assertEqual(ip('$v.a.r'), ['V.A.R'])
        self.assertEqual(ip('b $v.a.r'), ['b ', 'V.A.R'])
        self.assertEqual(ip('$v.a.r e'), ['V.A.R', ' e'])
        self.assertEqual(ip('b $v.a.r e'), ['b ', 'V.A.R', ' e'])

    def test05_Complex(self):
        self.assertEqual(ip('b $${v1}${v2}'), ['b ${v1}', 'V2'])
        self.assertEqual(ip('b $$v1$v2'), ['b $v1', 'V2'])
        self.assertEqual(ip('$v1$$v2$v3'), ['V1','$v2','V3'])
        self.assertEqual(ip('b${v1}$${v2}${v3}e'), ['b','V1','${v2}','V3','e'])

    def test06_Unicode(self):
        self.assertEqual(
            ip(il2('zatek${}$${k}${}dm')),
            [il2('zatek'), il2(''), il2('${k}'), il2(''), il2('dm')]
        )
        self.assertEqual(
            ip(il2('zatek $c $$k $ dm')),
            [il2('zatek '), il2('C'), il2(' $k $ dm')]
        )

    def test10_Multiline(self):
        self.assertEqual(
            list(interpolation_parser(('b\n$v\ne$x'),
                lambda s,l: s, lambda s,l: '{%s:%d}' % (s,l))),
            ['b\n', '{v:2}', '\ne', '{x:3}']
        )
        self.assertEqual(
            list(interpolation_parser(('b\n$v\ne$x'),
                lambda s,l:s, lambda s,l: '{%s:%d}' % (s,l), first_line=10)),
            ['b\n', '{v:11}', '\ne', '{x:12}']
        )
        self.assertEqual(
            list(interpolation_parser(('b\nc\nd\n'),
                lambda s,l: s, lambda s,l: '{%s:%d}' % (s,l))),
            ['b\nc\nd\n'],
        )

class TestHypertextize(unittest.TestCase):

    def test01_Email(self):
        self.assertEqual(
            list(hypertextize(
                'a b john@example.com c bill@microsoft.com d'
            )),[
             ('a b ',None),
             ('john@example.com','mailto:john@example.com'),
             (' c ', None),
             ('bill@microsoft.com', 'mailto:bill@microsoft.com'),
             (' d', None),
        ])

    def test02_EmailAndDomain(self):
        self.assertEqual(
            list(hypertextize(
                'a www.dat.cz b john@example.com https://www.uh.cz/'
            )),[
             ('a ',None),
             ('www.dat.cz', 'http://www.dat.cz'),
             (' b ', None),
             ('john@example.com','mailto:john@example.com'),
             (' ', None),
             ('https://www.uh.cz/', 'https://www.uh.cz/'),
        ])
