import unittest
from lib4.urls import *

class TestUrlPathJoin(unittest.TestCase):
    cases = [
        ('/a',      '',         '/a'),
        ('',        '',         ''),
        ('/a/y',    '/',        '/'),
        ('/a/y',    '/x',       '/x'),
        ('/',       'x',        '/x'),
        ('/',       'x',        '/x'),
        ('/',       './x',      '/x'),
        ('/a/',     'x',        '/a/x'),
        ('/a/y',    'x',        '/a/x'),
        ('/a/y',    './x',      '/a/x'),
        ('/a/y',    './',       '/a/'),
        ('/a/y',    '../a/',    '/a/'),
        ('/a/y',    './../a/',  '/a/'),
        #('/a/y',    '../../a/', '/a/'), ???
    ]
    def test(self):
        for b,p,r in self.cases:
            x = urlPathJoin(b,p)
            self.assertEqual(x,r,'urlPathJoin(%r,%r) == %r != %r' % (b,p,x,r))


class TestURL(unittest.TestCase):

    def test10_SimpleURL(self):
        u = URL('https://wm4.cz/info.cgi?a=1&b=2#T1')
        self.assertEqual(u, 'https://wm4.cz/info.cgi?a=1&b=2#T1')
        self.assertEqual(u.scheme, 'https')
        self.assertEqual(u.netloc, 'wm4.cz')
        self.assertEqual(u.path, '/info.cgi')
        self.assertEqual(u.query, 'a=1&b=2')
        self.assertEqual(u.target, 'T1')
        self.assertEqual(u.params, {'a':'1','b':'2'})

    def test11_AttributesAreReadonly(self):
        u = URL('https://wm4.cz/info.cgi?a=1&b=2#T1')
        self.assertRaises(AttributeError, setattr, u, 'scheme', 'http')
        self.assertRaises(AttributeError, setattr, u, 'netloc', 'http')
        self.assertRaises(AttributeError, setattr, u, 'path', 'http')
        self.assertRaises(AttributeError, setattr, u, 'query', 'http')
        self.assertRaises(AttributeError, setattr, u, 'target', 'http')
        self.assertRaises(AttributeError, setattr, u, 'params', 'http')

    def test12_URLWithParams(self):
        u = URL('http://wm4.cz/info.cgi?a=1#T1',params={'b':2}, target=32)
        self.assertEqual(u, 'http://wm4.cz/info.cgi?b=2#32')
        self.assertEqual(u.scheme, 'http')
        self.assertEqual(u.netloc, 'wm4.cz')
        self.assertEqual(u.path, '/info.cgi')
        self.assertEqual(u.query, 'b=2')
        self.assertEqual(u.target, '32')
        self.assertEqual(u.params, {'b':'2'}) # value 2 is encoded to '2'

    def test12_URLWithParamList(self):
        u = URL('http://wm4.cz/info.cgi?b=1&b=2#T1')
        self.assertEqual(u, 'http://wm4.cz/info.cgi?b=1&b=2#T1')
        self.assertEqual(u.scheme, 'http')
        self.assertEqual(u.netloc, 'wm4.cz')
        self.assertEqual(u.path, '/info.cgi')
        self.assertEqual(u.query, 'b=1&b=2')
        self.assertEqual(u.target, 'T1')
        self.assertEqual(u.params, {'b':['1','2']}) # value 2 is encoded to '2'

    def test12_URLWithExplicitParamList(self):
        u = URL('http://wm4.cz/info.cgi?a=1#T1',params={'b':[1,2]}, target=32)
        self.assertEqual(u, 'http://wm4.cz/info.cgi?b=1&b=2#32')
        self.assertEqual(u.scheme, 'http')
        self.assertEqual(u.netloc, 'wm4.cz')
        self.assertEqual(u.path, '/info.cgi')
        self.assertEqual(u.query, 'b=1&b=2')
        self.assertEqual(u.target, '32')
        self.assertEqual(u.params, {'b':['1','2']}) # value 2 is encoded to '2'

    def test13_PreparsedURL(self):
        u = URL(('https','wm4.cz','/info.cgi','a=1&b=2','T1'))
        self.assertEqual(u, 'https://wm4.cz/info.cgi?a=1&b=2#T1')
        self.assertEqual(u.scheme, 'https')
        self.assertEqual(u.netloc, 'wm4.cz')
        self.assertEqual(u.path, '/info.cgi')
        self.assertEqual(u.query, 'a=1&b=2')
        self.assertEqual(u.target, 'T1')
        self.assertEqual(u.params, {'a':'1','b':'2'})

    def test14_URLFromURL(self):
        u1 = URL(('https','wm4.cz','info.cgi','a=1&b=2','T1'))
        u2 = URL(u1)
        assert u1 is u2

    def test18_Hash(self):
        u1, u2 = URL('a'), URL('a')
        assert u1 is not u2
        self.assertEqual(hash(u1), hash(u2))

    def test20_Join(self):
        self.assertEqual(
            URL('http://wm4.cz/a/b/c?a=1#T1')/'d',
            URL('http://wm4.cz/a/b/d')
        )
        self.assertEqual(
            URL('http://wm4.cz/a/b/c?a=1#T1')/'d/',
            URL('http://wm4.cz/a/b/d/')
        )

    def test30_AddParams(self):
        self.assertEqual(
            URL('http://wm4.cz/a/b/c?a=1&b=2') % {'c':3},
            URL('http://wm4.cz/a/b/c?a=1&c=3&b=2')
        )
        self.assertEqual(
            URL('http://wm4.cz/a/b/c?a=1&b=2') % {'c':3, 'b':4},
            URL('http://wm4.cz/a/b/c?a=1&c=3&b=4')
        )

    def test31_ClearParams(self):
        self.assertEqual(
            URL('http://wm4.cz/a/b/c?a=1&b=2', params={}),
            URL('http://wm4.cz/a/b/c')
        )

    def test30_AddParamList(self):
        self.assertEqual(
            URL('http://wm4.cz/a/b/c?a=1&b=2') % {'c':[3,4]},
            URL('http://wm4.cz/a/b/c?a=1&c=3&c=4&b=2')
        )
        self.assertEqual(
            URL('http://wm4.cz/a/b/c?a=1&b=2') % {'c':[3,4], 'b':4},
            URL('http://wm4.cz/a/b/c?a=1&c=3&c=4&b=4')
        )


    def test40_RelativeURL(self):
        u = URL('a/b')
        self.failIf(u.scheme)
        self.failIf(u.netloc)
        self.failIf(u.query)
        self.failIf(u.target)
        self.assertEqual(u, 'a/b')
        u = URL('..')
        self.failIf(u.scheme)
        self.failIf(u.netloc)
        self.failIf(u.query)
        self.failIf(u.target)
        self.assertEqual(u, '..')
        u = URL('a/b?x=1#T')
        self.failIf(u.scheme)
        self.failIf(u.netloc)
        self.assertEqual(u.query, 'x=1')
        self.assertEqual(u.target, 'T')
        self.assertEqual(u, 'a/b?x=1#T')
        self.assertEqual(u % {'y':2}, 'a/b?y=2&x=1#T')


    relative_cases = [
        # URL                    base                   result
        ('http://web.cz/a',     'ftp://web.cz/aaa',     'http://web.cz/a'),
        ('http://web.cz/a',     'ftp://web.cz/aaa',     'http://web.cz/a'),
        ('http://web.cz/a',     'http://www.web.cz/a',  'http://web.cz/a'),
        #('http://web.cz/aaa',   '/aaa',                 'http://web.cz/aaa'),
        #('/aaa',                'http://web.cz/aaa',    '/aaa'),

        ('/a/b',        '/a/',      'b'),
        ('/a/',         '/a/b',     './'),
        ('/a/',         '/a/b/c',   '../'),
        ('/a/b',        '/a/c',     'b'),
        ('/a/b',        '/a',       'a/b'),
        ('/a/b/c',      '/a/',      'b/c'),
        ('/a/b/c',      '/x/',      '../a/b/c'),
        ('/a/b/c',      '/x/y',     '../a/b/c'),
        ('/a/b/c',      '/x/y/',    '../../a/b/c'),
        ('/a/b/',       '/a/b/c',   './'),
        ('/a/b',        '/a/b/c/d', '../../b'),
        ('/a/b',        '/a/b/c',   '../b'),

        ('http://web.cz/a/b',   'http://web.cz/a/b/c',  '../b'),
    ]

    def test41_RelativeTo(self):
        for o,b,r in self.relative_cases:
            ou = URL(o)
            bu = URL(b)
            o1 = URL(r).absoluteTo(bu)
            self.assertEqual(
                o1, ou, 'base=%r, rel=%r, ok=%r, bad=%r' % (b,r,o,o1)
            )
            r1 = ou.relativeTo(bu)
            self.assertEqual(
                r1, r, 'url=%r, base=%r, ok=%r, bad=%r' % (o,b,r, r1)
            )

