#!/usr/bin/env python2.2

# inc_string http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/426333
# http://cheeseshop.python.org/pypi/chardet
# http://www.nedbatchelder.com/code/modules/hyphenate.html
# http://trific.ath.cx/software/enca/ + pyenca

import codecs, re

__all__  = [
    'banner', 'unicode2ascii', 'fTemplate', 'ufTemplate', 'hypertextize',
    'interpolation_parser', 'guess_encoding',
]

def banner(text, ch='=', length=78):
    # http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/306863
    if text is None:
        return ch * length
    elif len(text) + 2 + len(ch)*2 > length:
        # Not enough space for even one line char (plus space) around text.
        return text
    else:
        remain = length - (len(text) + 2)
        prefix_len = remain / 2
        suffix_len = remain - prefix_len
        if len(ch) == 1:
            prefix = ch * prefix_len
            suffix = ch * suffix_len
        else:
            prefix = ch * (prefix_len/len(ch)) + ch[:prefix_len%len(ch)]
            suffix = ch * (suffix_len/len(ch)) + ch[:suffix_len%len(ch)]
    return prefix + ' ' + text + ' ' + suffix




def minimalize(s):
    n = ''
    for l in s.split('\n'):
        #if re.match( '^[-_]{2}.*', l ):
        #    if len(n) > 10: break
        if re.match('^>.*', l): continue
        n = n + l + ' '
    n = re.sub( "[ \n\t\r\f]+", " ", n )
    return n.strip()




def normalize_charset(cs):
    cs = cs.replace('-', '_')
    if cs.startswith('windows_'):
        cs = 'cp' + cs[8:]
    return cs



def unicode2ascii(us, replace=''):
    # see also:
    # https://svn.plone.org/svn/plone/CMFPlone/trunk/UnicodeNormalizer.py
    # http://www.unicode.org/reports/tr15/tr15-23.html
    # http://www.w3.org/TR/2004/WD-charmod-norm-20040225/
    # http://groups.google.com/group/comp.lang.python/browse_frm/thread/0ba554cf80455a08/77196f64a90ea9bc#77196f64a90ea9bc
    from unicodedata import normalize, combining
    us = unicode(us)
    s = []
    for c in normalize('NFKD',us):
        if combining(c):
            continue
        if ord(c) < 128:
            s.append(c)
        elif replace is not None:
            s.append(replace)
        else:
            raise UnicodeEncodeError('ascii',c,0,1,'cannot convert to ascii')
    return ''.join(s)











class ufTemplate:

    join = u''.join

    def __init__(self, *parts):
        self.callbacks = []
        self.output = []
        self.extend(parts)

    def write(self, *parts):
        self.extend(parts)

    def extend(self, parts):
        cb = self.callbacks
        out = self.output
        for p in parts:
            if isinstance(p, basestring):
                out.append(p)
            else:
                cb.append((len(out),p))
                out.append(None)

    def getStatic(self):
        if self.callbacks:
            return None
        else:
            return self.join(self.output)

    def __call__(self, *args, **kw):
        cbs = self.callbacks
        out = self.output[:]
        if cbs:
            for i,cb in cbs:
                out[i] = cb(*args, **kw)
        return self.join(out)


class fTemplate(ufTemplate):

    join = ''.join











def hypertextize(txt):
    global hypertextizeEmail, hypertextizeUrl
    try:
        ere = hypertextizeEmail
        ure = hypertextizeUrl
    except NameError:
        from validators import EMAIL_REGEX, DOMAIN_REGEX
        ere = hypertextizeEmail = re.compile(
            '((?:mailto:)?%s)' % EMAIL_REGEX()
        )
        ure = hypertextizeUrl = re.compile(
            '((?:(?:ftp|http|https)://)?' + DOMAIN_REGEX() + '[-?&=._/a-zA-Z0-9]*)'
        )

    for i, t in enumerate(ere.split(txt)):
        if i % 2:
            if t.startswith('mailto:'):
                n = t
            else:
                n = 'mailto:%s' % t
            yield t, n
        else:
            for j, u in enumerate(ure.split(t)):
                if j % 2:
                    if not u.startswith('ftp://') and \
                       not u.startswith('http://') and \
                       not u.startswith('https://'):
                        n = 'http://%s' % u
                    else:
                        n = u
                    yield u, n
                elif u:
                    yield u, None







def interpolation_parser(text, text_factory, subst_factory, first_line=1):
    global interpExpr, interpShort
    try:
        interpExpr, interpShort
    except NameError:
        interpExpr = re.compile(r"(?<!\$)\$\{(.+?)\}", re.M)
        interpShort = re.compile(r"(?<!\$)\$([a-zA-Z][a-zA-Z0-9_\.]*)")
    ln = first_line
    for i, p in enumerate(interpExpr.split(text)):
        if (i % 2) == 1:
            yield subst_factory(p, ln)
            ln += p.count('\n')
        else:
            for j,p in enumerate(interpShort.split(p)):
                if (j % 2) == 1:
                    yield subst_factory(p, ln)
                else:
                    if p:
                        yield text_factory(p.replace('$$','$'), ln)
                ln += p.count('\n')








encoding_matrix = {
   codecs.BOM_UTF8: 'utf_8',
   codecs.BOM_UTF16: 'utf_16',
   codecs.BOM_UTF16_BE: 'utf16_be',
   codecs.BOM_UTF16_LE: 'utf16_le',
}

def guess_encoding(data):
    """
    Given a byte string, guess the encoding.

    First it tries for UTF8/UTF16 BOM.

    Next it tries the standard 'UTF8', 'ISO-8859-1', and 'cp1252' encodings,
    Plus several gathered from locale information.

    The calling program *must* first call
        locale.setlocale(locale.LC_ALL, '')

    If successful it returns
        (decoded_unicode, successful_encoding)
    If unsuccessful it raises a ``UnicodeError``.
    """
    # also http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/163743
    import locale
    for bom, enc in encoding_matrix.items():
        if data.startswith(bom):
            return data.decode(enc), enc
    encodings = ['ascii', 'UTF-8']
    successful_encoding = None
    try:
        encodings.append(locale.nl_langinfo(locale.CODESET))
    except AttributeError:
        pass
    try:
        encodings.append(locale.getlocale()[1])
    except (AttributeError, IndexError):
        pass
    try:
        encodings.append(locale.getdefaultlocale()[1])
    except (AttributeError, IndexError):
        pass
    # latin-1
    encodings.append('ISO8859-1')
    encodings.append('cp1252')
    for enc in encodings:
        if not enc:
            continue
        try:
            decoded = unicode(data, enc)
            successful_encoding = enc
            break
        except (UnicodeError, LookupError):
            pass
    if successful_encoding is None:
        raise UnicodeError('Unable to decode input data. Tried the'
           ' following encodings: %s.' % ', '.join([repr(enc)
               for enc in encodings if enc]))
    else:
        if successful_encoding == 'ascii':
            # our default ascii encoding
            successful_encoding = 'ISO8859-1'
        return (decoded, successful_encoding)
