from properties import ComputedProperty

__all__ = [
    'AbstractTreeNode', 'TreeNodeWrapper', 'nodeListRootAndFilter',
    'treeFromNodeList', 'TreeTableCell', 'treeToHTable',
]


class AbstractTreeNode(object):
    '''Base class for tree nodes

       Concrete subclass should implement at least following attributes:

          'getParent()' -- reference to parent node or None

          'getChildren()' -- child nodes container (list, tuple, etc).

          'getNodeId()' -- returns node ID that is unique in a whole tree.

       Any subclass can redefine AbstractTreeNode descriptors if it
       has more efficient implementation for them.
    '''

    def getParent(self):
        '''Default implementation relies on parent attribute'''
        return self.parent

    def getChildren(self):
        '''Default implementation relies on children attribute'''
        return self.children

    def iterParents(self, include_self=True):
        '''Iterates over all node parents including node itself'''
        if not include_self:
            self = self.getParent()
        while self:
            yield self
            self = self.getParent()

    def iterParentsRev(self, include_self=True):
        '''Reversed parent iteration, i.e. starting with root node'''
        parents = list(self.iterParents(include_self))
        parents.reverse()
        return parents

    def getRoot(self):
        return list(self.iterParents())[-1]

    def iterDepthFirst(self, filter=None, include_self=True):
        '''DFS iterator

            'filter' -- filter function with signature filter(node).
            Subtree is pruned if filter returns True.
        '''
        if include_self and (filter is None or not filter(self)):
            yield self
        for n in self.getChildren():
            for x in n.iterDepthFirst(filter):
                yield x

    def iterBreadthFirst(self, include_self=True):
        '''BFS iterator'''
        # http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/231503
        if include_self:
            yield self
        last = self
        for node in self.iterBreadthFirst():
            for child in node.getChildren():
                yield child
                last = child
            if last is node:
                return

    def getNodeId(self):
        '''Unique node ID within whole tree'''
        return str(id(self))

    def findNode(self, nodeId):
        for node in self.iterBreadthFirst():
            if node.getNodeId() == nodeId:
                return node
        return None

    def treeLevel(self):
        '''Number of parent nodes'''
        parent = self.getParent()
        if parent:
            return parent.treeLevel + 1
        else:
            return 0
    treeLevel = ComputedProperty(treeLevel)

    def subtreeDepth(self):
        '''Length of longest branch in a subtree'''
        children = self.getChildren()
        if children:
            return max([n.subtreeDepth for n in children])+1
        else:
            return 1
    subtreeDepth = ComputedProperty(subtreeDepth)

    def nodeNum(self):
        '''Number of nodes in a subtree'''
        children = self.getChildren()
        if children:
            return sum([n.nodeNum for n in children])+1
        else:
            return 1
    nodeNum = ComputedProperty(nodeNum)

    def leafNum(self):
        '''Number of leaf nodes in a subtree'''
        children = self.getChildren()
        if children:
            return sum([n.leafNum for n in children])
        else:
            return 1
    leafNum = ComputedProperty(leafNum)




def no_filter(node):
    return False

class TreeNodeWrapper(AbstractTreeNode):
    '''Wrapper for creating views over existing tree

       View is also tree with original tree node is stored in `node'
       attribute. View is able to filter out some branches with given
       `filter' function.
    '''
    def __init__(self, node, parent=None, filter=no_filter, **kw):
        self.node = node
        self.parent = parent
        self.filter = filter
        for a,v in kw.items():
            setattr(self, a, v)

    def makeChildren(self):
        klass = self.__class__
        filt = self.filter
        return [
            klass(n, self, filt) for n in self.node.getChildren() if not filt(n)
        ]

    children = ComputedProperty(lambda s:s.makeChildren(), attrName='children')

    def getNodeId(self):
        return self.node.getNodeId()




def nodeListRootAndFilter(nodes):
    all_nodes = set()
    root = None

    for leaf in nodes:
        for j, node in enumerate(leaf.iterParentsRev()):
            if j == 0:
                if root is None:
                    root = node
                else:
                    if node is not root:
                        raise ValueError('nodes have multiple roots',root,node)
            all_nodes.add(node)

    def filter(node):
        return node not in all_nodes

    return root, filter


def treeFromNodeList(nodes, factory=TreeNodeWrapper):
    if not nodes:
        return None
    root, filter = nodeListRootAndFilter(nodes)
    return factory(root, filter=filter)








class TreeTableCell:
    node    = None
    colspan = None
    rowspan = None

    def __init__(self, **kw):
        self.__dict__.update(kw)

    def __repr__(self):
        return '<TreeTableCell colspan=%r rowspan=%r node=%r>' % (
            self.colspan, self.rowspan, self.node)

    def __eq__(self, other):
        if not isinstance(other, TreeTableCell):
            return False
        else:
            return (
                self.node, self.colspan, self.rowspan
            ) == (
                other.node, other.colspan, other.rowspan
            )

    def __ne__(self, other):
        return not self == other

def treeToHTable(tree, includeRoot=True):
    rows = []
    depthFirst = tree.iterDepthFirst()
    last = depthFirst.next()
    if includeRoot:
        row = [TreeTableCell(node=last,colspan=1,rowspan=last.leafNum)]
    else:
        row = []
    for n in depthFirst:
        if not None and last.treeLevel >= n.treeLevel:
            rows.append(row)
            row = []
        if not n.getChildren():
            colspan = tree.subtreeDepth - n.subtreeDepth - (
                n.treeLevel - tree.treeLevel - 1
            )
        else:
            colspan = 1
        row.append(TreeTableCell(node=n, colspan=colspan, rowspan=n.leafNum))
        last = n
    if row:
        rows.append(row)
    return rows
