# URI Template
# http://bitworking.org/projects/URI-Templates/
# http://pypi.python.org/pypi/uri/1.0
#
# rest2web/pythonutils/urlpath.py
#
# http://www.peterbe.com/plog/parametrize_url
# http://diveintopython.org/http_web_services/index.html
# http://www.voidspace.org.uk/python/articles/urllib2.shtml
# http://www.voidspace.org.uk/python/articles/authentication.shtml
# http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/491261
#
# [1462525] URI parsing library
# http://sourceforge.net/tracker/index.php?func=detail&aid=1462525&group_id=5470&atid=305470
#
# [1500504] Alternate RFC 3986 compliant URI parsing module
# http://sourceforge.net/tracker/index.php?func=detail&aid=1500504&group_id=5470&atid=305470

from urlparse import urlsplit, urlunsplit, urljoin
from urllib import unquote, quote

__all__ = [
    'URL','urlPathJoin','relativePath','decode_qs','encode_qs','encode_value',
]


class URL(str):

    def __new__(klass, url, params=None, target=None):
        parts  = ('scheme','netloc','path','query','target')

        if isinstance(url, klass):
            if params is None and target is None:
                return url
            parsed = (url.scheme,url.netloc,url.path,url.query,url.target)
            url = str(url)
        elif isinstance(url, (tuple,list)):
            parsed = url
            url = None
        else:
            url = str(url)
            parsed = urlsplit(url)

        if params is not None or target is not None:
            parsed = list(parsed)
            if params is not None:
                params = dict((a,encode_value(v)) for a,v in params.iteritems())
                parsed[3] = encode_qs(params)
            if target:
                parsed[4] = str(target)
            url = None

        else:
            params = parsed[3] and decode_qs(parsed[3]) or {}

        if url is None:
            url = urlunsplit(parsed)

        self = str.__new__(klass, url)

        for a,v in zip(parts, parsed):
            self.__dict__[a] = v
        self.__dict__['params'] = params

        return self
    scheme = property(lambda s:s.__dict__['scheme'])
    netloc = property(lambda s:s.__dict__['netloc'])
    path   = property(lambda s:s.__dict__['path'])
    query  = property(lambda s:s.__dict__['query'])
    target = property(lambda s:s.__dict__['target'])
    params = property(lambda s:s.__dict__['params'])

    def __div__(self, other):
        return URL(urljoin(self, other))

    def __mod__(self, params):
        p = self.params.copy()
        p.update(params)
        return self.__class__(self, params=p)

    def __sub__(self, other):
        return self.relativeTo(other)

    def relativeTo(self, base):
        if not isinstance(base, URL):
            base = URL(base)
        if self.scheme != base.scheme:
            return self
        if self.netloc != base.netloc:
            return self
        path = relativePath(base.path, self.path)
        return URL(path, params=self.params, target=self.target)

    def absoluteTo(self, base):
        if not isinstance(base, URL):
            base = URL(base)
        if self.scheme and self.scheme != base.scheme:
            return self
        if self.netloc and self.netloc != base.netloc:
            return self
        return base / self.path



def urlPathJoin(bpath, path):
    segments = bpath.split('/')[:-1] + path.split('/')
    if not path:
        return bpath
    if path.startswith('/'):
        return path
    if segments[-1] == '.':
        segments[-1] = ''
    while '.' in segments:
        segments.remove('.')
    while 1:
        i = 1
        n = len(segments) - 1
        while i < n:
            if (segments[i] == '..'
                and segments[i-1] not in ('', '..')):
                del segments[i-1:i+1]
                break
            i = i+1
        else:
            break
    if segments == ['', '..']:
        segments[-1] = ''
    elif len(segments) >= 2 and segments[-1] == '..':
        segments[-2:] = ['']
    return '/'.join(segments)


def relativePath(base, url):
    '''Convert absolute 'url' to be relative to 'base', if possible'''

    if base.endswith('/'):
        bDir = base[:-1]
        bFile = None
    else:
        bDir, bFile = base.rsplit('/',1)

    if url.endswith('/'):
        uDir = url[:-1]
        uFile = None
    else:
        uDir, uFile = url.rsplit('/',1)

    bParts = bDir.split('/')
    uParts = uDir.split('/')
    while bParts and uParts:
        if bParts[0] != uParts[0]:
            break
        bParts.pop(0)
        uParts.pop(0)

    parts = ['..'] * len(bParts)
    parts.extend(uParts)
    if uFile:
        parts.append(uFile)
    if parts:
        path = '/'.join(parts)
        if not uFile and not path.endswith('/'):
            path = path + '/'
        return path
    else:
        return './'


def decode_qs(qs, enc='utf-8'):
    dict = {}
    for name_value in [s2 for s1 in qs.split('&') for s2 in s1.split(';')]:
        if not name_value:
            continue
        nv = name_value.split('=', 1)
        if len(nv) == 1:
            name = unquote(nv[0].replace('+',' '))
            value = u''
        else:
            name = unquote(nv[0].replace('+', ' '))
            value = unicode(unquote(nv[1].replace('+', ' ')), enc)

        if name in dict:
            old = dict[name]
            if isinstance(old, list):
                old.append(value)
            else:
                dict[name] = [old,value]
        else:
            dict[name] = value
    return dict


def encode_value(v, enc='utf-8'):
    if isinstance(v, (list,tuple)):
        return [encode_value(i) for i in v]
    if isinstance(v, unicode):
        return v.encode(enc)
    else:
        return str(v)


def encode_qs(params, enc='utf-8'):
    qs = []
    for a,v in params.iteritems():
        if isinstance(v, (list,tuple)):
            for lv in v:
                qs.append('%s=%s' % (a,quote(encode_value(lv, enc))))
        else:
            qs.append('%s=%s' % (a,quote(encode_value(v, enc))))
    return '&'.join(qs)
